/*  Exile - IRC to Relational Database Gateway
 *  Copyright (C) 2002 Lucas Nussbaum <lucas@lucas-nussbaum.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: main.c 1333 2005-10-02 15:51:49Z sirvulcan $
 */

/* Exile - main file */

#include "exile.h"
#include "actions.h"
#include "conf.h"
#include "log.h"
#include "misc.h"
#include "send.h"
#include "sockutil.h"
#include "compat.h"
#include "memory.h"
#include "process.h"
#include "db.h"

/* Global variables */

int debug = 0;						  /* -d , debug mode = don't fork + log to stderr */

int verbose = 0;					  /* -v, verbose output */

/* Socket for talking to server */
int servsock = -1;
char *quitmsg;

char inbuf[BUFSIZE];

/* At what time were we started? */
time_t start_time;

extern char *PIDFilename;
extern char *RemoteServer;
extern int RemotePort;
extern char *LocalHost;
extern int LocalPort;
extern char *RemotePassword;
extern char *ServerNum;
extern char *ServerNumber;
extern char *ServerName;
extern char *ServerDesc;
extern char *BotNick;
extern char *BotIdent;
extern char *BotHostName;
extern char *BotRealName;
extern char *BotModes;
extern char *LogChannels;

/* display usage */
void usage()
{
	printf("\n   Syntax : exile\n");
	printf("         -v : enable verbose mode\n");
	printf("         -d : enable debugging mode\n");
	printf("         -V : display version and exit\n");
	exit(1);
}

/* Create our PID file and write the PID to it. */
void write_pidfile(void)
{
	FILE *pidfile;

	pidfile = fopen(PIDFilename, "w");
	if (pidfile)
	{
		fprintf(pidfile, "%d\n", (int) getpid());
		fclose(pidfile);
	}
	else
	{
		mylog_perror("Warning: cannot write to PID file %s", PIDFilename);
	}
}

/* Main routine */
int main(int argc, char **argv)
{
	char ch;
	int i;
	char *ch1, *ch2;
        char *chanb, *chan = NULL;
	/* record start time */
	start_time = time(NULL);
	/* Parse command line options */
	while ((ch = getopt(argc, argv, "dvV")) != -1)
	{
		switch (ch)
		{
		case 'd':
			debug = 1;
			verbose = 1;
			break;
		case 'v':
			verbose = 1;
			break;
		case 'V':
			printf("Exile v.%s\n", VERSION);
			exit(0);
			break;
		default:
			usage();
		}
	}

	/* Read configuration file; exit if there are problems. */
	if (!read_config())
		exit(-1);

	char ServerNumberf[3] = "A2";
	int ServerNumberint;
	ServerNumberint = atoi(ServerNum);
	inttobase64(ServerNumberf, ServerNumberint, 2);
	ServerNum  = ServerNumberf;

	/* Open logfile, and complain if we didn't. */
	if (open_log() < 0)
	{
		fprintf(stderr, "Warning: unable to open log file : %s\n",
				  strerror(errno));
		exit(1);
	}

	/* Detach ourselves if requested. */
	if (!debug)
	{
		if ((i = fork()) < 0)
		{
			perror("fork()");
			return -1;
		}
		else if (i != 0)
		{
			exit(0);
		}
		if (isatty(0) && isatty(1) && isatty(2))
		{
			close(0);
			close(1);
			close(2);
		}
	}
	write_pidfile();

	/* Announce ourselves to the logfile. */
	mylog("Exile %s starting up with %s support%s", VERSION,
#if defined (IRCD_IRCU11)
		 "IRCu 2.10.11.x",
#elif defined (IRCD_IRCU12)
		 "IRCu 2.10.12.x",
#elif defined (IRCD_NEFARIOUS4)
		"Nefarious IRCu 0.4.0",
#elif defined (IRCD_NEFARIOUS5)
		"Nefarious IRCu 0.5.0",
#elif defined (IRCD_ASUKA)
		"Asuka IRCu 1.2.1",
#else
		 "unknown (!)",
#endif
		 verbose ? (debug ? " (options: debug)" : " (options: verbose") :
		 "");
	/* I should init databases here */
	db_connect();
	ch1 = db_escape(ServerName);
	ch2 = db_escape(ServerDesc);
	db_query("INSERT INTO " TBL_SERV
				" (server, snumeric, comment, connecttime) values('%s', '%s', '%s', NOW())",
				ch1, ServerNum, ch2);
	db_addserver(ch1, db_insertid());
	db_addserver(ServerNum, db_insertid());
	/* connect to RemoteServer */
	servsock = conn(RemoteServer, RemotePort, LocalHost, LocalPort);
	if (servsock < 0)
	{
		fatal_perror("Can't connect to server");
		exit(-1);
	}
	send_cmd(NULL, "PASS %s", RemotePassword);
	send_cmd(NULL, "SERVER %s 1 %ld %ld %s %s]]] :%s", ServerName, start_time, start_time, "J10",
		ServerNum, ServerDesc);

	if (strlen(LogChannels) > 1) {
		int servid;
		char numeric[10];

		servid = db_getserver(ServerNum);

		send_cmd(NULL, "%s N %s 1 %ld %s %s %s AAAAAA %sAAA :%s", ServerNum, BotNick, start_time, BotIdent,
			BotHostName, BotModes, ServerNum, BotRealName);

#if defined(IRCD_NEFARIOUS4)||defined(IRCD_NEFARIOUS5)
                db_query
                        ("INSERT INTO " TBL_USER
                         "     (nick,   snumeric, account, acctimestamp, realname, hostname, sethost, fakehost, ipaddr, username, connecttime, servid) VALUES(\'%s\', \'%s\',   \'%s\',  \'%s\',       \'%s\',   \'%s\',   \'%s\',  \'%s\',   \'%s\', \'%s\', FROM_UNIXTIME(\'%ld\'), \'%d\')",
                               BotNick, ServerNum, "", "", BotRealName, BotHostName, "", "", "AAAAAA", BotIdent, start_time, servid);
#else
                db_query
                        ("INSERT INTO " TBL_USER
                         "     (nick,  snumeric  , account, acctimestamp, realname, hostname, ipaddr, username, connecttime,           servid) VALUES(\'%s\', \'%sAAA\', \'%s\',   \'%s\',       \'%s\',   \'%s\',   \'%s\', \'%s\',   FROM_UNIXTIME(\'%ld\'), \'%d\')",
                         BotNick, ServerNum, "", "", BotRealName, BotHostName, "AAAAAA", BotIdent, start_time, servid);
#endif

		sprintf(numeric, "%sAAA", ServerNum);
                db_addnick(BotNick, db_insertid());
                db_addnick(numeric, db_insertid());

	        for (chan = strtok_r(LogChannels, ",", &chanb);
	                chan;
	                chan = strtok_r(NULL, ",", &chanb)) {

			send_cmd(NULL, "%s B %s %ld %sAAA", ServerNum, chan, start_time, ServerNum);
			do_addusers(db_getchancreate(chan), BotNick, start_time);
		}
	}

        send_cmd(NULL, "%s EB", ServerNum);

	sgets2(inbuf, sizeof(inbuf), servsock);
	if (strnicmp(inbuf, "ERROR", 5) == 0)
	{
		/* Close server socket first to stop wallops, since the other
		 * server doesn't want to listen to us anyway */
		disconn(servsock);
		servsock = -1;
		fatal("Remote server returned: %s", inbuf);
	}
	/* We have a line left over from earlier, process it now */
	checkcommands();
	process();
	while (1)
	{
		i = (int) (long) sgets2(inbuf, sizeof(inbuf), servsock);
		if (i > 0)
		{
			checkcommands();
			process();
		}
		else if (i == 0)
		{
			int errno_save = errno;
			quitmsg = (char *) scalloc(BUFSIZE, 1);
			if (quitmsg)
			{
				snprintf(quitmsg, BUFSIZE, "Read error from server: %s",
							strerror(errno_save));
			}
			else
			{
				quitmsg = "Read error from server";
			}
			send_cmd(ServerName, "SQUIT %s :%s", ServerName, quitmsg);
			disconn(servsock);
			close_log();
			return 0;
		}
	}
}
